@@ -14,7 +14,7 @@ use strict;
 use IO::File;
 use Carp;
 
-our $VERSION = 0.98;
+our $VERSION = 1.0_1; # woo hoo!
 use Geo::ReadGRIB::PlaceIterator;
 
 my $LIB_DIR = "./";
@@ -122,6 +122,7 @@ sub openGrib {
         #     $self->{$_} = $head{$_};
     }
 
+
     # reduce date string to 'time' format
     my ( $yr, $mo, $day, $hr ) = unpack 'A4A2A2A2', $self->{d};
     $self->{TIME} = timegm( 0, 0, $hr, $day, $mo - 1, $yr - 1900 );
@@ -130,6 +131,11 @@ sub openGrib {
 
     $self->parseGDS( $head{GDS10} );
 
+    # if this isn't a lat/long grid we can't go on
+    if ( $self->{GRID_TYPE} != 0 ) {
+        croak "GDS byte 6 not 0: Only latitude/longitude grids are currently supported";
+    }
+
     $self->_getCatalog;
 
     return;
@@ -143,8 +149,8 @@ sub openGrib {
 #--------------------------------------------------------------------------
 sub getCatalogVerbose {
     my $self = shift;
-    $self->{ERROR} = "Method getCatalogVerbose() DEPRECATED and is now redundant  ";
-    $self->{ERROR} .= "in Geo::ReadGRIB V1.0 and above. Use getFullCatalog() instead";
+    $self->error( "Method getCatalogVerbose() DEPRECATED and is now redundant  "
+        . "in Geo::ReadGRIB V0.98 and above. Use getFullCatalog() instead" );
     $self->getFullCatalog();
     return 1;
 }
@@ -159,8 +165,8 @@ sub getCatalogVerbose {
 #--------------------------------------------------------------------------
 sub getCatalog {
     my $self = shift;
-    $self->{ERROR} = "Method getCatalog DEPRECATED and is no longer needed  ";
-    $self->{ERROR} .= "in Geo::ReadGRIB V1.0 and above";
+    $self->error( "Method getCatalog DEPRECATED and is no longer needed  "
+        . "in Geo::ReadGRIB V1.0 and above" );
     return 1;
 }
 
@@ -237,30 +243,235 @@ sub parseGDS {
 
     my @GDS = split /\s+/, $gds;
 
+    $self->{GRID_TYPE} = $GDS[5];
+
     my @slice = @GDS[ 6, 7 ];
-    $self->{Ni} = $self->toDecimal( \@slice );
+    $self->Ni( $self->toDecimal( \@slice ) );
 
     @slice = @GDS[ 8, 9 ];
-    $self->{Nj} = $self->toDecimal( \@slice );
+    $self->Nj( $self->toDecimal( \@slice ) );
 
     @slice = @GDS[ 10, 11, 12 ];
-    $self->{La1} = $self->toDecimal( \@slice ) / 1000;
+    $self->La1( $self->toDecimal( \@slice ) / 1000 );
 
     @slice = @GDS[ 13, 14, 15 ];
-    $self->{Lo1} = $self->toDecimal( \@slice ) / 1000;
+    $self->Lo1( $self->toDecimal( \@slice ) / 1000 );
+
+    my @rc_flags = split '', sprintf "%08b", $GDS[16];
+    
+    # if INCREMENTS_FLAG is true get increments from GDS else calculate
+    $self->{INCREMENTS_FLAG} = $rc_flags[0]; 
 
     @slice = @GDS[ 17, 18, 19 ];
-    $self->{La2} = $self->toDecimal( \@slice ) / 1000;
+    $self->La2( $self->toDecimal( \@slice ) / 1000 );
 
     @slice = @GDS[ 20, 21, 22 ];
-    $self->{Lo2} = $self->toDecimal( \@slice ) / 1000;
+    $self->Lo2( $self->toDecimal( \@slice ) / 1000 );
 
-    $self->{LaInc} = $self->calInc( $self->{La1}, $self->{La2}, $self->{Nj} );
-    $self->{LoInc} = $self->calInc( $self->{Lo1}, $self->{Lo2}, $self->{Ni} );
+    @slice = @GDS[ 23, 24 ];
+    if ( ($slice[0] == $slice[1] and $slice[0] == 255) and not $self->{INCREMENTS_FLAG} ) {
+        $self->LoInc( $self->calInc( $self->Lo1, $self->Lo2, $self->Ni ) );
+    }
+    else {
+        $self->LoInc( $self->toDecimal( \@slice ) / 1000 );
+    }
+
+    @slice = @GDS[ 25, 26 ];
+    if ( ($slice[0] == $slice[1] and $slice[0] == 255) and not $self->{INCREMENTS_FLAG} ) {
+        $self->LaInc = $self->calInc( $self->La1, $self->La2, $self->Nj );
+    }
+    else {
+        $self->LaInc( $self->toDecimal( \@slice ) / 1000 );
+    }
+
+    my @scan_mode_flags = split '', sprintf "%08b", $GDS[27];
+
+    # if SN_SCAN_FLAG true data runs in numeric order south-north
+    # elce data runs north-south
+    $self->sn_scan_flag( $scan_mode_flags[1] );
+
+    if ( $scan_mode_flags[0] ) {
+        croak "Scan mode -i not yet supported. Please contact the Geo::ReadGRIB maintainer to add support for GRIB files that scan east to west";
+    }
+
+    if ( $scan_mode_flags[2] ) {
+        croak "Scan mode (J,I) not yet supported. Please contact the Geo::ReadGRIB maintainer to add support for GRIB files where adjacent points are consecutive in the j direction (north/south)";
+    }
+
+    # Calculate effective Lo where thay are shifted west to 0 degrees.
+    # This will be used for finding data offset and for checking for 
+    # out of range args where ranges may cross long 0
+    $self->{eLo2} =  $self->Lo2;
+    $self->{eLo1} =  $self->Lo2 - ($self->Ni -1) * $self->LoInc;
+    $self->{Lo_SHIFT} = 0 - $self->{eLo1};
 
     return;
 }
 
+#--------------------------------------------------------------------------
+# sn_scan_flag( [flag] )
+# Getter/setter for flag
+#--------------------------------------------------------------------------
+sub sn_scan_flag {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{SN_SCAN_FLAG} = $flag if defined $flag;
+    return  $self->{SN_SCAN_FLAG};
+}
+
+#--------------------------------------------------------------------------
+# error( [error message] )
+# Getter/setter for errors
+#--------------------------------------------------------------------------
+sub error {
+    my $self = shift;
+    my $error = shift;
+
+    $self->{ERROR} = $error if defined $error;
+    return  $self->{ERROR};
+}
+
+#--------------------------------------------------------------------------
+# Lo1( [val] )
+# Getter/setter for Lo1
+#--------------------------------------------------------------------------
+sub Lo1 {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{Lo1} = $flag if defined $flag;
+    return  $self->{Lo1};
+}
+
+#--------------------------------------------------------------------------
+# Lo2( [flag] )
+# Getter/setter for Lo2
+#--------------------------------------------------------------------------
+sub Lo2 {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{Lo2} = $flag if defined $flag;
+    return  $self->{Lo2};
+}
+
+#--------------------------------------------------------------------------
+# LoInc( [flag] )
+# Getter/setter for LoInc
+#--------------------------------------------------------------------------
+sub LoInc {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{LoInc} = $flag if defined $flag;
+    return  $self->{LoInc};
+}
+
+#--------------------------------------------------------------------------
+# La1( [flag] )
+# Getter/setter for La1
+#--------------------------------------------------------------------------
+sub La1 {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{La1} = $flag if defined $flag;
+    return  $self->{La1};
+}
+
+#--------------------------------------------------------------------------
+# La2( [flag] )
+# Getter/setter for La2
+#--------------------------------------------------------------------------
+sub La2 {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{La2} = $flag if defined $flag;
+    return  $self->{La2};
+}
+
+#--------------------------------------------------------------------------
+# LaInc( [flag] )
+# Getter/setter for LaInc
+#--------------------------------------------------------------------------
+sub LaInc {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{LaInc} = $flag if defined $flag;
+    return  $self->{LaInc};
+}
+
+#--------------------------------------------------------------------------
+# Ni( [flag] )
+# Getter/setter for Ni
+#--------------------------------------------------------------------------
+sub Ni {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{Ni} = $flag if defined $flag;
+    return  $self->{Ni};
+}
+
+#--------------------------------------------------------------------------
+# Nj( [flag] )
+# Getter/setter for Nj
+#--------------------------------------------------------------------------
+sub Nj {
+    my $self = shift;
+    my $flag = shift;
+
+    $self->{Nj} = $flag if defined $flag;
+    return  $self->{Nj};
+}
+
+#--------------------------------------------------------------------------
+# clearError()
+# set errors undef
+#--------------------------------------------------------------------------
+sub clearError {
+    my $self = shift;
+    undef $self->{ERROR};
+}
+
+
+#--------------------------------------------------------------------------
+# validLo( 1ong )
+#
+# check that longs are in range and return true if they are, else false.
+#--------------------------------------------------------------------------
+sub validLo {
+
+    my $self = shift;
+    my $lo   = shift;
+
+    $lo += $self->{Lo_SHIFT};
+    if ( $lo > 360 ) {
+        $lo -= 360;
+    }
+ 
+    $lo /= $self->LoInc;
+    if ( $lo < 0 or $lo > $self->Ni ) {
+        return 0;
+    }
+
+    return 1;
+}
+
+#--------------------------------------------------------------------------
+# adjustLong( long )
+#
+# takes a long and returns a effective long adjusted to the shifted grid
+#--------------------------------------------------------------------------
+sub adjustLong {
+
+    my $self = shift;
+    my $l1   = shift;
+}
+
 #--------------------------------------------------------------------------
 # toDecimal()
 #
@@ -318,7 +529,7 @@ sub calInc {
     my $size;
     if ( $pts == 0 ) {
         $size = 0;
-        $self->{ERROR} = "calInc: \$pts = 0";
+        $self->error( "calInc: \$pts = 0" );
     }
     elsif ( $c1 < 0 or $c2 < 0 ) {
 
@@ -344,26 +555,51 @@ sub lalo2offset {
     my $lat  = shift;
     my $long = shift;
 
-    undef $self->{ERROR};
+    
+    my $thislong = 0;
+#    $long += $self->{Lo_SHIFT};
+
+    $self->clearError;
 
     # First check if values are out of range...
-    if ( $lat > $self->{La1} or $lat < $self->{La2} ) {
-        $self->{ERROR} = "lalo2offset(): LAT >$lat< out of range ";
-        $self->{ERROR} .= "$self->{La1} to $self->{La2}";
-        return;
+    my ($lo1, $lo2, $la1, $la2) = ($self->Lo1, $self->Lo2, $self->La1, $self->La2, );
+    if ( $self->sn_scan_flag ) {
+        if ( $lat < $self->La1 or $lat > $self->La2 ) {
+            $self->error( "lalo2offset(): LAT >$lat< out of range $la1 to $la2" );
+            return -1;
+        }
+    }
+    else {
+        if ( $lat > $self->La1 or $lat < $self->La2 ) {
+            $self->error( "lalo2offset(): LAT >$lat< out of range $la1 to $la2" );
+            return -1;
+        }
     }
 
-    if ( $long < $self->{Lo1} or $long > $self->{Lo2} ) {
-        $self->{ERROR} = "lalo2offset(): LONG: >$long< out of range ";
-        $self->{ERROR} .= "$self->{Lo1} to $self->{Lo2}";
-        return;
+    if ( not $self->validLo( $long ) ) {
+        $self->error( "lalo2offset(): LONG: >$long< out of range $lo1 to $lo2" );
+        return -1;
     }
 
-    my $out =
-      ( ( $self->{La1} - $lat ) / $self->{LaInc} ) * $self->{Ni} +
-      ( ( $long - $self->{Lo1} ) / $self->{LoInc} );
+    my $out;
+    
+    if ( $self->sn_scan_flag ) {
+        $thislong = $long + $self->{Lo_SHIFT};
 
-    return sprintf "%d", $out;
+        if ( $thislong > 360 ) {
+            $thislong -= 360;
+        }
+
+        $out =  ( ( ( $lat - $self->La1  ) / $self->LaInc ) * $self->Ni ) 
+                + ( ($thislong ) / $self->LoInc );
+    }
+    else {
+        $out =  ( ( $self->La1 - $lat ) / $self->LaInc ) * $self->Ni 
+                + ( ($long - $self->Lo1 ) / $self->LoInc );
+    }
+
+    return int( $out );
+#    return sprintf "%d", $out;
 }
 
 #--------------------------------------------------------------------------
@@ -404,51 +640,50 @@ sub extractLaLo {
         push @types, $type_s;
     }
     else {
-        $self->{ERROR} = "ERROR extractLaLo() \$types required";
+        $self->error( "ERROR extractLaLo() \$types required" );
         return;
     }
 
     my @times = sort keys %{ $self->{catalog} };
 
     # First see that requested values are in range...
-
+    my ($lo1, $lo2, $la1, $la2) = ($self->Lo1, $self->Lo2, $self->La1, $self->La2, );
     if ( not $lat1 >= $lat2 or not $long1 <= $long2 ) {
-        $self->{ERROR} = "ERROR extractLaLo() ";
-        $self->{ERROR} .= "require: lat1 >= lat2 and long1 <= long2";
+        $self->error( "ERROR extractLaLo() require: lat1 >= lat2 and long1 <= long2" );
         return;
     }
 
     if ( not defined $time ) {
-        $self->{ERROR} = "ERROR extractLaLo() \$time is required ";
+        $self->error( "ERROR extractLaLo() \$time is required " );
         return;
     }
 
     if ( $time < $self->{TIME} or $time > $self->{LAST_TIME} ) {
-        $self->{ERROR} = "ERROR extractLaLo() \$time \"$time\" out of range ";
-        $self->{ERROR} .= scalar gmtime( $times[0] ) . " ($times[0]) to ";
-        $self->{ERROR} .=
-          scalar gmtime( $times[-1] ) . " ($times[-1])";
+        $self->error( "ERROR extractLaLo() \$time \"$time\" out of range " 
+        .  scalar gmtime( $times[0] ) . " ($times[0]) to " 
+        . scalar gmtime( $times[-1] ) . " ($times[-1])" );
         return;
     }
 
-    if (   $lat1 > $self->{La1}
-        or $lat1 < $self->{La2}
-        or $lat2 > $self->{La1}
-        or $lat2 < $self->{La2} )
-    {
-        $self->{ERROR} = "extractLaLo(): LAT >$lat1 or $lat2< out of range ";
-        $self->{ERROR} .= "$self->{La1} to $self->{La2}";
-        return;
+    if ( $self->sn_scan_flag ) {
+        if (   $lat1 < $self->La1 or $lat1 > $self->La2
+            or $lat2 < $self->La1 or $lat2 > $self->La2 )
+        {
+            $self->error( "extractLaLo(): LAT >$lat1 or $lat2< out of range $la1 to $la2" );
+            return;
+        }
+    }
+    else {
+        if (   $lat1 > $self->La1 or $lat1 < $self->La2
+            or $lat2 > $self->La1  or $lat2 < $self->La2 )
+        {
+            $self->error( "extractLaLo(): LAT >$lat1 or $lat2< out of range $la1 to $la2" );
+            return;
+        }
     }
 
-    if (   $long1 < $self->{Lo1}
-        or $long1 > $self->{Lo2}
-        or $long2 < $self->{Lo1}
-        or $long2 > $self->{Lo2} )
-    {
-        $self->{ERROR} =
-          "extractLaLo(): LONG: >$long1 or $long2 < out of range ";
-        $self->{ERROR} .= "$self->{Lo1} to $self->{Lo2}";
+    if ( not $self->validLo( $long1 ) or not $self->validLo( $long2 ) ) {
+        $self->error( "extractLaLo(): LONG: >$long1 or $long2 < out of range $lo1 to $lo2" );
         return;
     }
 
@@ -456,17 +691,7 @@ sub extractLaLo {
 
     # if time is given, use nearest time in catalog...
     if ( defined $time ) {
-        for ( my $i = 0, my $j = 1 ; $j <= @times ; $i++, $j++ ) {
-            if ( $time >= $times[$i] and $time <= $times[$j] ) {
-                if ( ( $time - $times[$i] ) <= ( $times[$j] - $time ) ) {
-                    $self->{THIS_TIME} = $times[$i];
-                }
-                else {
-                    $self->{THIS_TIME} = $times[$j];
-                }
-                last;
-            }
-        }
+        $self->findNearestTime( $time );
     }
 
     my ( $offset, $dump, $record, $lo, $la );
@@ -475,6 +700,11 @@ sub extractLaLo {
     for my $type ( @types ) {
         $record = $self->{catalog}->{$tm}->{$type};
 
+        if ( not defined $record ) {
+            $self->error( "extractLaLo(): Not a valid type: $type" );
+            return 1;
+        }
+
         my $tmp = $self->tempfile();
         my $cmd =
           "\"$LIB_DIR\"/wgrib.exe \"$self->{fileName}\" -d $record -nh -o $tmp";
@@ -482,9 +712,11 @@ sub extractLaLo {
         my $F = IO::File->new( $tmp ) or croak "Can't open temp file";
 
         $dump = "";
-        for ( $lo = $long1 ; $lo <= $long2 ; $lo += $self->{LoInc} ) {
-            for ( $la = $lat1 ; $la >= $lat2 ; $la -= $self->{LaInc} ) {
+        for ( $lo = $long1 ; $lo <= $long2 ; $lo += $self->LoInc ) {
+            for ( $la = $lat1 ; $la >= $lat2 ; $la -= $self->LaInc ) {
                 $offset = $self->lalo2offset( $la, $lo );
+#                $la = sprintf "%.4f", $la;
+#                $lo = sprintf "%.4f", $lo;
                 seek $F, $offset * 4, 0;
                 read $F, $dump, 4;
                 $dump = unpack "f", $dump;
@@ -524,41 +756,32 @@ sub extract {
 
     my $offset = $self->lalo2offset( $lat, $long );
 
-    $time = 0 unless defined $time;
+    if ( $offset < 0 ) {
+        # do not reset error, there will be one from lalo2offset()
+        return 1;
+    }
 
-    undef $self->{ERROR};
+    $time = 0 unless defined $time;
 
     if (   $time != 0 and $time < $self->{TIME} 
        or $time > $self->{LAST_TIME} ) {
 
         my @times = sort keys %{ $self->{catalog} };
-        $self->{ERROR} = "ERROR extract() \$time \"$time\" out of range ";
-        $self->{ERROR} .= scalar gmtime( $times[0] ) . " ($times[0]) to ";
-        $self->{ERROR} .=
-          scalar gmtime( $times[-1] ) . " ($times[-1])";
+        $self->error( "ERROR extract() \$time \"$time\" out of range " 
+        . scalar gmtime( $times[0] ) . " ($times[0]) to " 
+        . scalar gmtime( $times[-1] ) . " ($times[-1])" );
         return 1;
     }
 
     # If a time is given find nearest in catalog
     if ( defined $time ) {
-        my @times = sort keys %{ $self->{catalog} };
-        for ( my $i = 0, my $j = 1 ; $j <= @times ; $i++, $j++ ) {
-            if ( $time >= $times[$i] and $time <= $times[$j] ) {
-                if ( ( $time - $times[$i] ) <= ( $times[$j] - $time ) ) {
-                    $self->{THIS_TIME} = $times[$i];
-                }
-                else {
-                    $self->{THIS_TIME} = $times[$j];
-                }
-
-                last;
-            }
-        }
+        $self->findNearestTime( $time );
     }
+
     my ( $record, $cmd, $res, $dump );
 
     unless ( defined $self->{v_catalog}->{$type} ) {
-        $self->{ERROR} = "extract() Type not found: $type";
+        $self->error( "extract() Type not found: $type" );
         return 1;
     }
 
@@ -568,6 +791,8 @@ sub extract {
     # If record is alredy in $self->{data} use that
     # else go to disk...
 
+    my $plit = Geo::ReadGRIB::PlaceIterator->new();
+
     $dump = "";
     foreach my $tm ( sort keys %{ $self->{catalog} } ) {
         if ( $time != 0 ) {
@@ -576,6 +801,11 @@ sub extract {
  
         $record = $self->{catalog}->{$tm}->{$type};
 
+        if ( not defined $record ) {
+            $self->error( "extract(): Not a valid type: $type" );
+            return 1;
+        }
+
         my $tmp = $self->tempfile();
         $cmd = "\"$LIB_DIR\"/wgrib.exe \"$self->{fileName}\" -d $record -nh -o $tmp";
         $res = qx($cmd);
@@ -590,11 +820,42 @@ sub extract {
         print gmtime($tm) . ": $self->{v_catalog}->{$type}  $dump\n"
           if $self->{DEBUG};
         $self->{data}->{$tm}->{$lat}->{$long}->{$type} = $dump;
+        $plit->addData( $tm, $lat, $long, $type, $dump );
         close $F;
         unlink $tmp;
         last if $time != 0;
     }
-    return;
+    return $plit;
+}
+
+#--------------------------------------------------------------------------
+# fineNearestTime( time )
+# 
+# Find nearest time in catalog...
+#--------------------------------------------------------------------------
+sub findNearestTime {
+
+    my $self = shift;
+    my $time = shift;
+
+    my @times = sort keys %{ $self->{catalog} };
+
+    if ( 1 == @times ) {
+        $self->{THIS_TIME} = $times[0];
+    }
+    else {
+        for ( my $i = 0, my $j = 1 ; $j <= @times ; $i++, $j++ ) {
+            if ( $time >= $times[$i] and $time <= $times[$j] ) {
+                if ( ( $time - $times[$i] ) <= ( $times[$j] - $time ) ) {
+                    $self->{THIS_TIME} = $times[$i];
+                }
+                else {
+                    $self->{THIS_TIME} = $times[$j];
+                }
+                last;
+            }
+        }
+    }
 }
 
 #--------------------------------------------------------------------------
@@ -616,11 +877,11 @@ sub getDataHash {
 #--------------------------------------------------------------------------
 # getError()
 #
-# returns error string from $self->{ERROR}
+# returns error string from $self->error
 #--------------------------------------------------------------------------
 sub getError {
     my $self = shift;
-    return defined $self->{ERROR} ? $self->{ERROR} : undef;
+    return defined $self->error ? $self->error : undef;
 }
 
 #--------------------------------------------------------------------------
@@ -672,13 +933,12 @@ sub getParam {
             $param = $self->{$arg};
         }
         else {
-            $self->{ERROR} = "getParam(): ";
-            $self->{ERROR} .= "$arg - Undefined or unpublished parameter";
+            $self->error( "getParam(): $arg - Undefined or unpublished parameter" );
             return 1;
         }
     }
     else {
-        $self->{ERROR} = "getParam(): Usage: getParam(param_name)";
+        $self->error( "getParam(): Usage: getParam(param_name)" );
         return 1;
     }
 
@@ -708,8 +968,7 @@ sub show {
             $param = $self->{$arg};
         }
         else {
-            $self->{ERROR} =
-              "show(): $arg - Undefined or unpublished parameter";
+            $self->error( "show(): $arg - Undefined or unpublished parameter" );
             return 1;
         }
     }
@@ -747,9 +1006,12 @@ __END__
 
 =head1 NAME
 
+
+
 Geo::ReadGRIB - Perl extension that gives read access to GRIB 1 "GRIdded
 Binary" format Weather data files.
 
+
 =head1 SYNOPSIS
 
   use Geo::ReadGRIB;
@@ -762,7 +1024,7 @@ Binary" format Weather data files.
   
   print $w->show(); 
   
-  $w->extract(data_type, lat, long, time);
+  $plit = $w->extract(data_type, lat, long, time);
 
   # or 
 
@@ -778,7 +1040,7 @@ Binary" format Weather data files.
       $latitude   = $place->lat;
       $longitude  = $place->long;
       $data_types = $place->types; # an array ref of type names
-      $data       = $place->data;
+      $data       = $place->data( data_type );
 
       # do something with each place in the extracted rectangular area
   }
@@ -796,20 +1058,28 @@ Binary" format Weather data files.
 
 =head1 DESCRIPTION
 
-Geo::ReadGRIB is an object Perl module that provides read access to data
+Geo::ReadGRIB is an Perl module that provides read access to data 
 distributed in GRIB files. Specifically, I wrote it to access NOAA Wavewatch 
-III marine weather model forecasts which are packaged as GRIB. 
+III marine weather model forecasts. As of version 0.98_1 Geo::ReadGRIB is known to 
+support Canadian Meteorological Center's GEM model GRIB files. It may support
+many other GRIB variants which use a rectangular lat/long grid but they have not 
+been tested. Please notify the maintainers and let them know if it does or doesn't 
+support your files.
 
-This version introduces the Geo::ReadGRIB::PlaceIterator class. PlaceIterator
-objects are returned by extractLaLo() and can be used for an ordered traversal
-of the extracted data for a given time. This greatly simplifies map image 
-creation and other data analysis tasks. See L<Geo::ReadGRIB::PlaceIterator>
+Version 0.98 introduced the Geo::ReadGRIB::PlaceIterator class. PlaceIterator
+objects are returned by extractLaLo() and extract() and can be used for an 
+ordered traversal of the extracted data for a given time. This greatly simplifies 
+map image creation and other data analysis tasks. See L<Geo::ReadGRIB::PlaceIterator>
 documentation and demo programs in the examples directory.
 
 Wavewatch III GRIB's can currently be found under 
 
 ftp://polar.ncep.noaa.gov/pub/waves/
 
+CMC GRIB datasets currently listed at
+
+http://www.weatheroffice.gc.ca/grib/High-resolution_GRIB_e.html
+
 GRIB stands for "GRIdded Binary" and it's a format developed by the World
 Meteorological Organization (WMO) for the exchange of weather product 
 information. See for example 
@@ -827,10 +1097,7 @@ and should install in the same location as ReadGRIB.pm. ReadGRIB will search
 for wgrib.exe at run time and die if it can't find it.
 
 wgrib.c is known to compile and install correctly with Geo::ReadGRIB on 
-FreeBSD, LINUX and Windows. In all cases the compiler was gcc and on Windows 
-ActivePerl and nmake or Straberry Perl and dmake were used and the CC=gcc option 
-was used with Makefile.PL I've also been able to compile wgrib.c with gcc on 
-Solaris Sparc and i386.
+all common platforms. See the CPAN Testers reports for this module for details.
 
 wgrib.exe creates a file called wgrib.tmp.XXXXXXXXX in the local directory where 
 the X's are random chars. The id that runs a program using Geo::ReadGRIB needs
@@ -925,29 +1192,33 @@ Extracted data is also retained in the ReadGRIB object data structure.
 Since extractLaLo() needs only one call to wgrib and one temp file open,
 this is faster than using extract() to get the same data points one at a time.
 
-=item $object->extract(data_type, lat, long, I<time>);
+=item $plit = $object->extract(data_type, lat, long, I<time>);
 
 Extracts forecast data for given type and location. I<time> is optional.
 Extracts data for all times in file unless a specific time is given 
 in epoch seconds.
 
-lat and long will be in the range -90 to 90 degrees lat and 0 to 359 degrees
-long. Longitude in GRIB files is 0 to 359 east. If you have degrees in 
-longitude west you will need to convert it first. If lat or long is out of
-range for the current file an error will be set ( see getError() ).
+lat and long will be in the range 90 to -90 degrees lat and 0 to 360 degrees
+long. If lat or long is out of range for the current file an error will be 
+set ( see getError() ).
 
 time will be in epoch seconds as returned, for example, by Time::Local. If the 
 time requested is in the range of times in the file but not one of the exact 
 times in the file, the nearest existing time will be used. An error will be set
 if time is out of range.
 
-type will be one of the data types in the data or an error is set.
+data_type will be one of the data types in the GRIB file or an error is set.
+
+Returns a L<Geo::ReadGRIB::PlaceIterator> object containing the extracted data
+which can be used to iterate through the data in order sorted by lat and then long.
+
+Extracted data is also retained in the ReadGRIB object data structure.
 
 =item $object->getError();
 
 Returns string messages for the last error set. If no error is set getError()
-returns undef. Any method that sets errors will clear errors first when called.
-It's good practice to check errors after an extract().
+returns undef. Only the most recent error will be show. It's good practice to 
+check errors after actions, especially while developing applications.
 
 =item $object->getDataHash();
 
@@ -971,6 +1242,14 @@ getFullCatalog(). and sets an error.
 
 =back
 
+=head1 BUGS AND LIMITATIONS
+
+There are no known bugs in this module. Please report problems through
+
+http://rt.cpan.org
+
+or contact Frank Cox, <frank.l.cox@gmail.com> Patches are welcome.
+
 =head1 SEE ALSO
 
 For more on wgrib.c see 
